/* 
 * enPathia
 * Eneso Tecnología de Adaptación, S.L. 
 */

#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <libintl.h>
#include <locale.h>
#include <stdlib.h>
#include "edecor.h"
#include "ehid.h"
#include "esingle.h"
#include "calibration.h"
#include "clicker.h"
#include "enpathia.h"
#include "help.h"
#include "hotkeys.h"
#include "icon.h"
#include "keyboard.h"
#include "menu.h"
#include "misc.h"
#include "profile.h"
#include "unlock.h"
#include "users.h"
#include "window.h"

static gchar*	force_profile	= NULL;

static GOptionEntry entries [] =
{
	{ "disable-window", 'd', 0, G_OPTION_ARG_NONE, &disable_main_window, "Disable main window", NULL },
	{ "user-profile", 'p', 0, G_OPTION_ARG_STRING, &force_profile, "Force user profile", NULL },
	{ NULL }
};

static void
terminate	(void);

static gboolean
hid_idle_search_cb	(gpointer data);
static gboolean
hid_idle_receive_cb	(gpointer data);

static void
ehid_connected_cb		(guint vid, guint pid);
static void
ehid_disconnected_cb	(void);
static void
ehid_message_cb			(gchar* message, guint n_bytes);

static void
esingle_instance_cb		(void);



	int
main (int argc, char* argv [])
{
	GOptionContext*	option_context;
	gchar*			data_file;

	/* Check if there is another instance running */
	if (!esingle_init ("Eneso.enPathia.Single", esingle_instance_cb))
		return 0;

	/* Parse the commandline */
	option_context = g_option_context_new ("");
	g_option_context_add_main_entries (option_context, entries, NULL);
	g_option_context_add_group (option_context, gtk_get_option_group (TRUE));

	g_option_context_parse (option_context, &argc, &argv, NULL);
	g_option_context_free (option_context);

	/* Hotkeys */
	init_hotkeys ();

	/* GTK+ */
	gtk_init (&argc, &argv);

#ifdef __MACH__
	gtk_osx_app = g_object_new (GTK_TYPE_OSX_APPLICATION, NULL);
	gtk_osxapplication_ready (gtk_osx_app);
#endif	/* __MACH__ */

	printf ("GTK+ init\n");

	/* Find the data and config directories */
	find_data_dir ();
	find_config_dir ();

	/* Decoration */
	data_file = get_data_file ("/images/decoration/");
	edecor_init (data_file);
	g_free (data_file);

	/* Window icon */
	data_file = get_data_file ("/icons/enpathia.ico");
	gtk_window_set_default_icon_from_file (data_file, NULL);
	g_free (data_file);

	/* Load the configuration */
	load_config_file ();
	if (force_profile != NULL)
		load_profile (force_profile);
	get_screen_size ();

	/* Notification icon */
	create_icon ();

	/* Menu and clicker */
	init_actions ();
	create_menu ();
	set_menu_size (current_profile.config.menu_size);
	create_clicker ();
	set_clicker_size (current_profile.config.menu_size);

	/* Keyboard */
	create_keyboard ();
	set_keyboard_size (current_profile.config.keyb_size);

	/* Create the windows */
	create_help ();
	create_users_dialog ();
	load_calibration_window ();
	load_unlock_window ();
	load_main_window (
			current_profile.main_window.width,
			current_profile.main_window.height);

	/* HID communications */
	ehid_init ("enpathia", 0xEE50, 0x0101, 9, 9,
			ehid_connected_cb, ehid_disconnected_cb, ehid_message_cb);

	/* Show main window */
	if (current_profile.main_window.show)	
	{
		show_main_window (
				current_profile.main_window.x,
				current_profile.main_window.y);
	}

	update_main_window ();

	/* Main loop */
	g_atexit (terminate);
	gtk_main ();

	return 0;
}

	static void
terminate (void)
{
	save_current_profile ();

	destroy_icon ();
	destroy_actions ();
	destroy_keyboard ();
	destroy_help ();

	edecor_terminate ();
	ehid_terminate ();
	esingle_terminate ();

	ehotkey_unregister (100);
	ehotkey_terminate ();
}

	static gboolean
hid_idle_search_cb	(gpointer data)
{
	ehid_search ();

	return !device_connected;
}

	static gboolean
hid_idle_receive_cb	(gpointer data)
{
	ehid_receive ();

	return device_connected;
}

	static void 
ehid_connected_cb (guint vid, guint pid)
{
	device_connected = TRUE;
	sensor_connected = TRUE;
	menu_disabled = FALSE;

	change_icon (1);
	update_main_window ();

	unlock ();

	g_idle_add_full (G_PRIORITY_LOW, hid_idle_receive_cb, NULL, NULL);
}

	static void
ehid_disconnected_cb (void)
{
	device_connected = FALSE;
	sensor_connected = FALSE;

	change_icon (0);
	update_main_window ();

	if (unlocking)
		stop_unlock ();
	if (calibrating)
		stop_calibration ();
	if (showing_menu)
		hide_menu ();
	if (showing_clicker)
		hide_clicker ();
	if (showing_keyboard)
		hide_keyboard ();

#ifdef G_OS_UNIX
#ifndef __MACH__
	g_idle_add_full (G_PRIORITY_LOW, hid_idle_search_cb, NULL, NULL);
#endif
#endif
}

	static void
ehid_message_cb (gchar* message, guint n_bytes)
{
	handle_message (message, n_bytes);
}

	static void
esingle_instance_cb (void)
{
	show_main_window (
			current_profile.main_window.x, current_profile.main_window.y);
}
